@@ -79,8 +79,8 @@ class MatchGroup:
 
 
 def debug(*a: Any, **kw: Any) -> None:
-    kw['file'] = sys.stderr
-    print(*a, **kw)
+    from kittens.tui.loop import debug_write
+    debug_write(*a, **kw)
 
 
 class Delegate:
@@ -222,7 +222,6 @@ def zsh_output_serializer(ans: Completions) -> str:
             desc = desc[:truncate_point_for_length(desc, max_desc_len - 2)]
             desc += '…'
 
-        word = f'\x1b[32m{word}\x1b[39m'
         if multiline:
             ans = f'{word}\n  {desc}'
         else:
@@ -264,8 +263,9 @@ def zsh_output_serializer(ans: Completions) -> str:
                 lines.append('shift words')
                 lines.append('(( CURRENT-- ))')
             lines.append(f'_normal -p "{ans.delegate.precommand}"')
-    # debug('\n'.join(lines))
-    return '\n'.join(lines)
+    result = '\n'.join(lines)
+    # debug(result)
+    return result
 
 
 @output_serializer
@@ -478,7 +478,7 @@ def complete_icat_args(ans: Completions, opt: Optional[OptionDict], prefix: str,
     from .guess_mime_type import guess_type
 
     def icat_file_predicate(filename: str) -> bool:
-        mt = guess_type(filename)
+        mt = guess_type(filename, allow_filesystem_access=True)
         if mt and mt.startswith('image/'):
             return True
         return False
