@@ -23,7 +23,7 @@ class Version(NamedTuple):
 
 appname: str = 'kitty'
 kitty_face = '🐱'
-version: Version = Version(0, 22, 0)
+version: Version = Version(0, 22, 1)
 str_version: str = '.'.join(map(str, version))
 _plat = sys.platform.lower()
 is_macos: bool = 'darwin' in _plat
@@ -186,3 +186,11 @@ def read_kitty_resource(name: str) -> bytes:
     except ImportError:
         from importlib_resources import read_binary  # type: ignore
     return read_binary('kitty', name)
+
+
+def website_url(doc_name: str = '') -> str:
+    if doc_name:
+        doc_name = doc_name.rstrip('/')
+        if doc_name:
+            doc_name += '/'
+    return f'https://sw.kovidgoyal.net/kitty/{doc_name}'
