@@ -11,7 +11,7 @@ AddressFamily = PopenType = Socket = StartupCtx = None
 SessionTab = SessionType = LayoutType = SpecialWindowInstance = None
 MarkType = RemoteCommandType = CoreTextFont = FontConfigPattern = None
 KeyEventType = ImageManagerType = KittyCommonOpts = HandlerType = None
-GRT_t = GRT_a = GRT_d = GRT_f = GRT_m = GRT_o = None
+GRT_t = GRT_a = GRT_d = GRT_f = GRT_m = GRT_o = GRT_C = None
 ScreenSize = KittensKeyActionType = MouseEvent = AbstractEventLoop = None
 TermManagerType = LoopType = Debug = GraphicsCommandType = None
 
