@@ -12,8 +12,8 @@ from functools import partial
 from gettext import gettext as _
 from itertools import chain
 from typing import (
-    Any, Callable, Deque, Dict, Iterable, List, Optional, Pattern, Sequence,
-    Tuple, Union
+    Any, Callable, Deque, Dict, Iterable, List, NamedTuple, Optional, Pattern,
+    Sequence, Tuple, Union
 )
 
 from .child import ProcessDesc
@@ -33,7 +33,7 @@ from .fast_data_types import (
     set_titlebar_color, set_window_padding, set_window_render_data,
     update_window_title, update_window_visibility, viewport_for_window
 )
-from .keys import keyboard_mode_name
+from .keys import keyboard_mode_name, mod_mask
 from .notify import NotificationCommand, handle_notification_cmd
 from .options.types import Options
 from .rgb import to_color
@@ -72,6 +72,12 @@ class PipeData(TypedDict):
     text: str
 
 
+class ClipboardPending(NamedTuple):
+    where: str
+    data: str
+    truncated: bool = False
+
+
 class DynamicColor(IntEnum):
     default_fg, default_bg, cursor_color, highlight_fg, highlight_bg = range(1, 6)
 
@@ -337,7 +343,7 @@ class Window:
         self.tab_id = tab.id
         self.os_window_id = tab.os_window_id
         self.tabref: Callable[[], Optional[TabType]] = weakref.ref(tab)
-        self.clipboard_control_buffers = {'p': '', 'c': ''}
+        self.clipboard_pending: Optional[ClipboardPending] = None
         self.destroyed = False
         self.geometry: WindowGeometry = WindowGeometry(0, 0, 0, 0, 0, 0)
         self.needs_layout = True
@@ -524,6 +530,11 @@ class Window:
         get_boss().close_window(self)
 
     def send_text(self, *args: str) -> bool:
+        '''
+        @ac:misc: Send the specified text to the active window
+
+        For details, see :sc:`send_text`.
+        '''
         mode = keyboard_mode_name(self.screen)
         required_mode_, text = args[-2:]
         required_mode = frozenset(required_mode_.split(','))
@@ -560,6 +571,7 @@ class Window:
         get_boss().child_monitor.set_iutf8_winid(self.id, on)
 
     def on_mouse_event(self, event: Dict[str, Any]) -> bool:
+        event['mods'] = event.get('mods', 0) & mod_mask
         ev = MouseEvent(**event)
         self.current_mouse_event_button = ev.button
         action = get_options().mousemap.get(ev)
@@ -772,10 +784,29 @@ class Window:
     def send_cmd_response(self, response: Any) -> None:
         self.screen.send_escape_code_to_child(DCS, '@kitty-cmd' + json.dumps(response))
 
-    def clipboard_control(self, data: str) -> None:
+    def clipboard_control(self, data: str, is_partial: bool = False) -> None:
         where, text = data.partition(';')[::2]
+        if is_partial:
+            if self.clipboard_pending is None:
+                self.clipboard_pending = ClipboardPending(where, text)
+            else:
+                self.clipboard_pending = self.clipboard_pending._replace(data=self.clipboard_pending[1] + text)
+                if len(self.clipboard_pending.data) > 8 * 1024 * 1024:
+                    log_error('Discarding part of too large OSC 52 paste request')
+                    self.clipboard_pending = self.clipboard_pending._replace(data='', truncated=True)
+            return
+
         if not where:
-            where = 's0'
+            if self.clipboard_pending is not None:
+                text = self.clipboard_pending.data + text
+                where = self.clipboard_pending.where
+                try:
+                    if self.clipboard_pending.truncated:
+                        return
+                finally:
+                    self.clipboard_pending = None
+            else:
+                where = 's0'
         cc = get_options().clipboard_control
         if text == '?':
             response = None
@@ -797,22 +828,13 @@ class Window:
             except Exception:
                 text = ''
 
-            def write(key: str, func: Callable[[str], None]) -> None:
-                if text:
-                    if ('no-append' in cc or
-                            len(self.clipboard_control_buffers[key]) > 1024*1024):
-                        self.clipboard_control_buffers[key] = ''
-                    self.clipboard_control_buffers[key] += text
-                else:
-                    self.clipboard_control_buffers[key] = ''
-                func(self.clipboard_control_buffers[key])
-
             if 's' in where or 'c' in where:
                 if 'write-clipboard' in cc:
-                    write('c', set_clipboard_string)
+                    set_clipboard_string(text)
             if 'p' in where:
                 if 'write-primary' in cc:
-                    write('p', set_primary_selection)
+                    set_primary_selection(text)
+        self.clipboard_pending = None
 
     def manipulate_title_stack(self, pop: bool, title: str, icon: Any) -> None:
         if title:
@@ -827,21 +849,30 @@ class Window:
 
     # mouse actions {{{
     def mouse_click_url(self) -> None:
+        '@ac:mouse: Click the URL under the mouse'
         click_mouse_url(self.os_window_id, self.tab_id, self.id)
 
     def mouse_click_url_or_select(self) -> None:
+        '@ac:mouse: Click the URL under the mouse only if the screen has no selection'
         if not self.screen.has_selection():
             self.mouse_click_url()
 
     def mouse_selection(self, code: int) -> None:
+        '''
+        @ac:mouse: Manipulate the selection based on the current mouse position
+
+        For examples, see :ref:`conf-kitty-mouse.mousemap`
+        '''
         mouse_selection(self.os_window_id, self.tab_id, self.id, code, self.current_mouse_event_button)
 
     def paste_selection(self) -> None:
+        '@ac:mouse: Paste the current primary selection'
         txt = get_boss().current_primary_selection()
         if txt:
             self.paste(txt)
 
     def paste_selection_or_clipboard(self) -> None:
+        '@ac:mouse: Paste the current primary selection or the clipboard if no selection is present'
         txt = get_boss().current_primary_selection_or_clipboard()
         if txt:
             self.paste(txt)
@@ -905,6 +936,7 @@ class Window:
     # actions {{{
 
     def show_scrollback(self) -> None:
+        '@ac:cp: Show scrollback in a pager like less'
         text = self.as_text(as_ansi=True, add_history=True, add_wrap_markers=True)
         data = self.pipe_data(text, has_wrap_markers=True)
         get_boss().display_scrollback(self, data['text'], data['input_line_number'])
@@ -916,6 +948,7 @@ class Window:
         self.screen.paste_bytes(text)
 
     def paste(self, text: Union[str, bytes]) -> None:
+        '@ac:cp: Paste the specified text into the current window'
         if text and not self.destroyed:
             if isinstance(text, str):
                 text = text.encode('utf-8')
@@ -932,6 +965,7 @@ class Window:
             self.screen.paste(text)
 
     def copy_to_clipboard(self) -> None:
+        '@ac:cp: Copy the selected text from the active window to the clipboard'
         text = self.text_for_selection()
         if text:
             set_clipboard_string(text)
@@ -945,6 +979,7 @@ class Window:
         ).encode('ascii')
 
     def copy_or_interrupt(self) -> None:
+        '@ac:cp: Copy the selected text from the active window to the clipboard, if no selection, send Ctrl-C'
         text = self.text_for_selection()
         if text:
             set_clipboard_string(text)
@@ -952,10 +987,12 @@ class Window:
             self.write_to_child(self.encoded_key(KeyEvent(key=ord('c'), mods=GLFW_MOD_CONTROL)))
 
     def copy_and_clear_or_interrupt(self) -> None:
+        '@ac:cp: Copy the selected text from the active window to the clipboard and clear selection, if no selection, send Ctrl-C'
         self.copy_or_interrupt()
         self.screen.clear_selection()
 
     def pass_selection_to_program(self, *args: str) -> None:
+        '@ac:cp: Pass the selected text from the active window to the specified program'
         cwd = self.cwd_of_child
         text = self.text_for_selection()
         if text:
@@ -965,30 +1002,37 @@ class Window:
                 open_url(text, cwd=cwd)
 
     def scroll_line_up(self) -> None:
+        '@ac:sc: Scroll up by one line'
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_LINE, True)
 
     def scroll_line_down(self) -> None:
+        '@ac:sc: Scroll down by one line'
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_LINE, False)
 
     def scroll_page_up(self) -> None:
+        '@ac:sc: Scroll up by one page'
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_PAGE, True)
 
     def scroll_page_down(self) -> None:
+        '@ac:sc: Scroll down by one page'
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_PAGE, False)
 
     def scroll_home(self) -> None:
+        '@ac:sc: Scroll to the top of the scrollback buffer'
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_FULL, True)
 
     def scroll_end(self) -> None:
+        '@ac:sc: Scroll to the bottom of the scrollback buffer'
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_FULL, False)
 
     def toggle_marker(self, ftype: str, spec: Union[str, Tuple[Tuple[int, str], ...]], flags: int) -> None:
+        '@ac:mk: Toggle the current marker on/off'
         from .marks import marker_from_spec
         key = ftype, spec
         if key == self.current_marker_spec:
@@ -1009,14 +1053,23 @@ class Window:
         self.current_marker_spec = key
 
     def remove_marker(self) -> None:
+        '@ac:mk: Remove a previously created marker'
         if self.current_marker_spec is not None:
             self.screen.set_marker()
             self.current_marker_spec = None
 
     def scroll_to_mark(self, prev: bool = True, mark: int = 0) -> None:
+        '@ac:mk: Scroll to the next or previous mark of the specified type'
         self.screen.scroll_to_next_mark(mark, prev)
 
     def signal_child(self, *signals: int) -> None:
+        '''
+        @ac:misc: Send the specified SIGNAL to the foreground process in the active window
+
+        For example::
+
+            map F1 signal_child SIGTERM
+        '''
         pid = self.child.pid_for_cwd
         if pid is not None:
             for sig in signals:
